%% Statistiques des fluctuations de (X_*-\theta).
% Comme "fluctuations", cette fonction prend trois entrées : le paramètre
% "theta" et le nombre de simulations "N" de l'expérience statistique qu'on
% est en train de considérer, et le nombre "Nsim" de simulations à utiliser
% pour la méthode de Monte-Carlo. Elle renvoie un 2-vecteur "quantiles"
% dont les entrées sont les quantiles (estimés) à resp. 5 % et à 95 % de la
% loi de (X_*-\theta), ainsi que la valeur (estimée) "esperance" de
% l'espérance de cette loi.
function [quantiles, esperance] = fluctuations_compact (theta, N, Nsim)
simulations = zeros (1, Nsim);
% Tout en stockant les valeurs simulées en mémoire, nous allons aussi
% calculer leur somme, stockée dans "somme".
somme = 0;
for i = 1 : Nsim
    Xetoile = 0;
    for j = 1 : N
        Xj = theta * rand ();
        if Xj > Xetoile
            Xetoile = Xj;
        end
    end
    simulation = Xetoile - theta;
    simulations (i) = simulation;
    somme = somme + simulation;
end
% Pour calculer les quantiles empiriques des fluctuations, nous avons
% besoin de trier les valeurs obtenues de la plus petite à la plus grande.
% Nous appliquons ici un algorithme de tri naïf, où nous allons à chaque
% fois chercher la valeur la plus petite restant dans le vecteur
% "simulations". On stocke la liste triée dans "simulations_triees".
simulations_triees = zeros (1, Nsim);
for i = 1 : Nsim
    % "record" sera la valeur la plus petite restant dans "simulations", et
    % "recordman" sa position.
    record = 0;
    recordman = 0;
    for j = 1 : Nsim
        if simulations (j) < record
            record = simulations (j);
            recordman = j;
        end
    end
    % On rentre la plus grande valeur trouvée dans "simulations_trouvées",
    % et dans "simulations" on la remplace par une valeur qui ne pourra
    % plus être le record.
    simulations_triees (i) = record;
    simulations (recordman) = 0;
end
% On regarde les quantiles à 5 % et à 95 % des simulations, resp. appelés
% "quantile5" et "quantile95" ; ainsi que leur espérance, appelée
% "esperance".
quantile5 = simulations_triees (ceil (Nsim * 5 / 100));
quantile95 = simulations_triees (ceil (Nsim * 95 / 100));
esperance = somme / Nsim;
quantiles = [quantile5, quantile95];
return
end