%% État du réseau électrique.
% Cette fonction simule et trace l'évolution de l'état du réseau électrique au
% cours de la journée.
function elec_graph
global tdeb tfin taupan taurep ELEC_PANNE ELEC_OK PANNE REPARE FIN
% Initialisation ds paramètres globaux.
init_intermittences ();
% On commence par allouer de la mémoire pour stocker les résultats en vue de
% leur tracé. À cause de la méthode de tracé choisie dans ce code, on ne sait
% pas à l'avance de combien de place on aura besoin. Nous allons appeler
% "alloc_result" la place réservée pour les résultats, celle-ci étant
% susceptible d'évoluer au cours du programme. À vue de nez, le cycle «
% préparation - préchauffage - panne » fera que nous aurons à tracer deux paires
% de points tous les taupan+taurep environ, donc nous allons allouer à peu près
% une fois et demi la quantité de mémoire que cela suggère. En outre, nous avons
% besoin dans tous les cas que "alloc_result" soit au moins égal à 3, pour
% pouvoir tracer sans risque le point initial et la premère paire de points.
alloc_result = ceil (1.5 * (tfin - tdeb) / (taurep + taupan) * 4);
if alloc_result < 3
  alloc_result = 3;
end
% "t_graph" est destiné à stocker les valeurs des temps remarquables pour le
% tracé du graphique, et "elec_graph" les valeurs associées de l'état de
% l'usine.
t_graph = zeros (alloc_result, 1);
elec_graph = zeros (alloc_result, 1);
% "t" est la date du temps courant, initialisée à t_deb.
t = tdeb;
% "elec" est l'état du réseau électrique, qu'on initialise aléatoirement selon
% la loi précisée par l'énoncé.
if rand < taupan / (taurep + taupan)
  elec = ELEC_PANNE;
else
  elec = ELEC_OK;
end
% On stocke la valeur initiale de l'état du réseau dans les vecteurs de
% résultats. Il nous faudra maintenir un indice "i" qui désigne le numéro de la
% case qu'on est en train de remplir.
i = 1;
t_graph(i) = t;
elec_graph(i) = elec;
% Nous lançons maintenant la simulation de l'évolution de l'état du réseau.
% Comme nous ne savons pas à l'avance combien d'évènements vont se produire, il
% faut passer par une boucle "while".
while true
  % On calcule le temps d'attente "attente" avant que l'état du réseau
  % électrique bascule. Il faut distinguer s'il s'agit de passer de la panne à
  % la réparation ou du fonctionnement à la panne, puisque la loi exponentielle
  % n'a pas le même paramètre selon le cas. On utilise aussi un marqueur
  % "phenomene" qui nous dit quel évènement se produit à l'issue du temps
  % d'attente.
  switch elec
    case ELEC_OK
      phenomene = PANNE;
      attente = taupan * (-log (rand ()));
    case ELEC_PANNE
      phenomene = REPARE;
      attente = taurep * (-log (rand ()));
  end
  % On met à jour le temps. Mais attention : si le nouveau temps dépasse t_fin,
  % c'est qu'on a en fait atteint l'heure de fermeture ! Auquel cas on indique
  % ce phénomène dans le marqueur dédié, et on modifie "t" en conséquence.
  t = t + attente;
  if t > tfin
    phenomene = FIN;
    t = tfin;
  end
  % On inscrit le point suivant du graphique, juste avant qu'il se passe quelque
  % chose.
  i = i + 1;
  t_graph(i) = t;
  elec_graph(i) = elec;
  % Selon ce qui vient de se passer (indiqué par le marqueur "phenomene"), on
  % met à jour l'état du réseau électrique. Si le marqueur indique que la
  % simulation est finie, il faut sortir de la boucle.
  switch phenomene
    case PANNE
      elec = ELEC_PANNE;
    case REPARE
      elec = ELEC_OK;
    case FIN
      break
  end
  % On trace maintenant le point suivant du graphique, juste après qu'il s'est
  % passé quelque chose.
  i = i + 1;
  t_graph(i) = t;
  elec_graph(i) = elec;
  % Nous ne voulons pas risquer de remplir les vecteurs de résultats au-delà de
  % leur taille, car cela est une mauvaise pratique de programmation (même si en
  % l'occurrence Matlab sait plus ou moins gérer cela) : nous allons donc
  % vérifier qu'il nous reste assez de place pour écrire les deux prochaines
  % valeurs de résultats, en prévision du prochain passage dans la boucle. Dans
  % le cas contraire, il faudra augmenter la mémoire allouée pour les vecteurs
  % de résultats. De combien ? Comme l'opération de réallocation de mémoire est
  % couteuse en temps, il vaut mieux ne pas y procéder trop souvent ; on va donc
  % prendre de la marge en prenant le double la mémoire nécessaire à court
  % terme.
  if i + 2 > alloc_result
    alloc_result = 2 * (i + 2);
    % Les opérations ci-dessous ne sont pas d'authentiques affectations de
    % valeurs : c'est en fait la façon dont on dit à Matlab d'augmenter la
    % taille des vecteurs.
    t_graph (alloc_result) = 0;
    elec_graph (alloc_result) = 0;
  end
end
% On choisit d'afficher le résultat dans la fenêtre no 1, qu'on efface au
% préalable.
figure (1)
clf
% On trace le graphique (en précisant la couleur ainsi que l'épaisseur de la
% ligne). Attention à n'utiliser que les indices des vecteurs effectivement
% concernés par le tracé.
plot (t_graph(1 : i), elec_graph (1 : i), 'red', 'LineWidth', 2);
% Cette ligne est pour fixer les valeurs extrêmes de la fenêtre d'affichage
% indépendamment de son contenu.
axis ([tdeb, tfin, ELEC_PANNE, ELEC_OK]);
return
end
