%% État du réseau et de l'usine.
% Cette fonction simule et trace l'évolution jointe de l'état du réseau
% électrique et de l'usine au cours de la journée. Je ne commente que les ajouts
% par rapport à "elec_graph".
function elec_usine_graph
% Une nouvelle macro est à prendre en compte : "CHAUD", qui sert à indiquer
% quand le phénomène observé est la fin du précauffage.
global tdeb tfin taupan taurep Tpre ELEC_PANNE ELEC_OK...
  USINE_ARRET USINE_PRECH USINE_FONCT PANNE CHAUD REPARE FIN
init_intermittences ();
% On prévoit aussi un peu plus de place pour les résultats, vu qu'il faudra
% aussi tracer une paire de points à la fin du préchauffage.
alloc_result = ceil (1.5 * (tfin - tdeb) / (taurep + taupan) * 6);
if alloc_result < 3
  alloc_result = 3;
end
t_graph = zeros (alloc_result, 1);
elec_graph = zeros (alloc_result, 1);
% "usine_graph" va stocker l'état de l'usine aux différents points du tracé.
usine_graph = zeros (alloc_result, 1);
t = tdeb;
% Il faut initialiser non seulement l'état du réseau au début de la journée,
% mais aussi, en conséquence, celui de l'usine. Par ailleurs, comme le processus
% n'est plus markovien, on va avoir besoin de se souvenir de certains éléments
% du passé : cela est assuré par le marqueur "phenomene_prec", qui nous dit
% quelle est la dernière chose qui s'est produite. Une petite réflexion nous
% fait comprendre que la façon correcte d'initialiser "phenomene_prec" est
% "PANNE" si l'électricité est en panne en début de journée, resp. "REPARE" si
% elle fonctionne.
if rand < taupan / (taurep + taupan)
  elec = ELEC_PANNE;
  usine = USINE_ARRET;
  phenomene_prec = PANNE;
else
  elec = ELEC_OK;
  usine = USINE_PRECH;
  phenomene_prec = REPARE;
end
% On trace le premier point du graphique, en n'oubliant pas que cette fois-ci il
% faut aussi stocker les résultats pour l'état de l'usine.
i = 1;
t_graph(i) = t;
elec_graph(i) = elec;
usine_graph(i) = usine;
while true
  switch elec
    case ELEC_PANNE
      phenomene = REPARE;
      attente = taurep * (-log (rand ()));
    case ELEC_OK
      phenomene = PANNE;
      attente = taupan * (-log (rand ()));
  end
  % Dans le cas où le phénomène précédent était la réparation de l'électricité
  % et qu'aucune panne ne s'est produite pendant la durée T_pre, le prochain
  % phénomène qui se produit est en fait que les machines sont chaudes et que
  % l'usine va passer en régime de fonctionnement. Dans ce cas, en vertu de la
  % propriété de Markov, nous oublierons complètement la valeur exacte simulée
  % du temps d'attente et retiendrons simplement qu'il ne s'est rien passé avant
  % T_pre.
  if phenomene_prec == REPARE && attente > Tpre
    phenomene = CHAUD;
    attente = Tpre;
  end
  t = t + attente;
  if t > tfin
    phenomene = FIN;
    t = tfin;
  end
  i = i + 1;
  t_graph(i) = t;
  elec_graph(i) = elec;
  usine_graph(i) = usine;
  % Cette fois-ci, il y a quatre phénomènes différents qui peuvent se produire ;
  % et surtout, lorsqu'un phénomène se produit, il peut aussi y avoir l'état de
  % l'usine qui change.
  switch phenomene
    case PANNE
      elec = ELEC_PANNE;
      usine = USINE_ARRET;
    case CHAUD
      usine = USINE_FONCT;
    case REPARE
      elec = ELEC_OK;
      usine = USINE_PRECH;
    case FIN
      break
  end
  i = i + 1;
  t_graph(i) = t;
  elec_graph(i) = elec;
  usine_graph(i) = usine;
  if i + 2 > alloc_result
    alloc_result = 2 * (i + 2);
    t_graph (alloc_result) = 0;
    elec_graph (alloc_result) = 0;
  end
  % On met à jour "phenomene_prec" avant de repartir dans la boucle.
  phenomene_prec = phenomene;
end
figure (1)
clf
% On utilise la commande "hold on" pour éviter que la seconde commande "plot"
% n'efface la première courbe tracée. (Puis on utilisera "hold off" à la fin
% pour ré-éteindre cette fonctionnalité).
hold on
plot (t_graph(1 : i), elec_graph (1 : i), 'red', 'LineWidth', 2);
% Il faut aussi tracer le graphique de l'état de l'usine.
plot (t_graph(1 : i), usine_graph (1 : i), 'cyan', 'LineWidth', 2);
hold off
% La fenêtre d'affichage est plus grande que dans le cas précédent, afin qu'on
% puisse voir l'état de l'usine.
axis ([tdeb, tfin, ELEC_PANNE, USINE_FONCT]);
return
end
