%% Productions journalières conditionnelles couplées.
% Cette fonction simule de manière couplée l'espérance conditionnelle de la
% production pour l'ancienne chaine (conditionnellement à ce qui se passe
% jusqu'au temps t_fin-T_pre) et l'espérance conditionnelle de la production
% pour la nouvelle chaine (conditionnellement, cette fois-ci, à ce qui se passe
% jusqu'au temps t_fin-T_pre') ; le couplage étant, comme dans
% "usine_prod_coupl", d'utiliser la même simulation du réseau électrique pour
% les deux situations. La fonction renvoie les deux espérances conditionnelles
% respectives calculées sous la forme du 2-vecteur "Eproductions". Je ne
% commente que les nouveautés par rapport aux programmes "usine_prod_cond" et
% "usine_prod_coupl".
function Eproductions = usine_prod_cond_coupl
global tdeb tfin taupan taurep Tpre Tpre_neo ELEC_PANNE ELEC_OK ...
  USINE_ARRET USINE_PRECH USINE_FONCT PANNE DEMICHAUD CHAUD REPARE FIN D
t = tdeb;
if rand < taupan / (taurep + taupan)
  elec = ELEC_PANNE;
  usine = USINE_ARRET;
  usine_neo = USINE_ARRET;
  phenomene_prec = PANNE;
else
  elec = ELEC_OK;
  usine = USINE_PRECH;
  usine_neo = USINE_PRECH;
  phenomene_prec = REPARE;
end
% On introduit un marqueur "demifin" qui indique si on a arrêté la simulation
% pour l'ancienne chaine de production.
demifin = false;
production = 0;
production_neo = 0;
while true
  switch elec
    case ELEC_PANNE
      phenomene = REPARE;
      attente = taurep * (-log (rand ()));
    case ELEC_OK
      phenomene = PANNE;
      attente = taupan * (-log (rand ()));
  end
  if phenomene_prec == REPARE && attente > Tpre_neo
    phenomene = DEMICHAUD;
    attente = Tpre_neo;
  elseif phenomene_prec == DEMICHAUD && attente > Tpre - Tpre_neo
    phenomene = CHAUD;
    attente = Tpre - Tpre_neo;
  end
  t = t + attente;
  % Cette fois-ci, il y a deux temps de fin : le premier pour arrêter la
  % simulation pour l'ancienne chaine, à t_fin-T_pre, puis le second pour
  % arrêter la simulation pour la nouvelle chaine, à t_fin-T_pre' ; et ce n'est
  % qu'à ce second instant qu'on sortira de la boucle. En fait, nous n'allons
  % pas traiter le premier temps de fin comme une étape de notre simulation :
  % nous allons juste indiquer qu'il vient de se produire par un marqueur, et
  % faire à part les calculs relatifs à l'ancienne chaine pour l'étape au cours
  % de laquelle on arrête sa simulation.
  if ~demifin && t > tfin - Tpre
    % Lorsqu'on termine la simulation pour l'ancienne chaine, il faut savoir, le
    % cas échéant, depuis combien de temps elle était en préchauffage, ce qu'on
    % indique par "tprech". Il faut aussi, s'il y a lieu, ajouter à la
    % production de la chaine le nombre de boulons produits au cours de la
    % dernière étape de simulation la concernant. La variable "demiattente" sert
    % à indiquer combien de temps a réellement duré la dernière étape de la
    % simulation pour l'ancienne chaine.
    demiattente = attente - (t - (tfin - Tpre));
    switch phenomene_prec
      case REPARE
        tprech = demiattente;
      case DEMICHAUD
        tprech = Tpre_neo + demiattente;
      case CHAUD
        production = production + demiattente * D;
    end
    demifin = true;
  end
  % Moment où la simulation est complètement terminée.
  if t > tfin - Tpre_neo
    phenomene = FIN;
    attente = attente - (t - (tfin - Tpre_neo));
    t = tfin - Tpre_neo;
  end
  % Attention à ne pas augmenter la production pour l'ancienne chaine si sa
  % simulation est déjà terminée…
  if ~demifin && usine == USINE_FONCT
    production = production + attente * D;
  end
  if usine_neo == USINE_FONCT
    production_neo = production_neo + attente * D;
  end
  switch phenomene
    case PANNE
      elec = ELEC_PANNE;
      % On ne touche plus à l'état de l'ancienne chaine après la fin de sa
      % simulation : en effet, nous aurons besoin de savoir dans quel état
      % l'ancienne chaine était à la fin de sa simulation pour calculer
      % l'espérance conditionnelle de sa production résiduelle.
      if ~demifin
        usine = USINE_ARRET;
      end
      usine_neo = USINE_ARRET;
    case DEMICHAUD
      usine_neo = USINE_FONCT;
    case CHAUD
      if ~demifin
        usine = USINE_FONCT;
      end
    case REPARE
      elec = ELEC_OK;
      if ~demifin
        usine = USINE_PRECH;
      end
      usine_neo = USINE_PRECH;
    case FIN
      break
  end
  phenomene_prec = phenomene;
end
% On calcule l'espérance de la production résiduelle pour l'ancienne chaine,
% appelée "Eresidu".
switch usine
  case USINE_ARRET
    Eresidu = 0;
  case USINE_PRECH
    Eresidu = D * taupan * exp (-Tpre / taupan) * (exp (tprech / taupan) - 1);
  case USINE_FONCT
    Eresidu = D * taupan * (1 - exp (-Tpre / taupan));
end
% On calcule l'espérance de la production résiduelle pour la nouvelle chaine,
% appelée "Eresidu_neo".
switch usine_neo
  case USINE_ARRET
    Eresidu_neo = 0;
  case USINE_PRECH
    % Dans le cas où la nouvelle chaine était en préchauffage à l'heure
    % t_fin-T_pre', la durée depuis laquelle elle préchauffait peut être
    % retrouvée dans "attente".
    Eresidu_neo = ...
      D * taupan * exp (-Tpre_neo / taupan) * (exp (attente / taupan) - 1);
  case USINE_FONCT
    Eresidu_neo = D * taupan * (1 - exp (-Tpre_neo / taupan));
end
% On en déduit les espérances conditionnelles totales respectives, appelées
% resp. "Eproduction" et "Eproduction_neo", et les agrège dans "Eproductions".
Eproduction = production + Eresidu;
Eproduction_neo = production_neo + Eresidu_neo;
Eproductions = [Eproduction; Eproduction_neo];
return
end
