%% 'Simulation du processus.
% Cette fonction simule le processus de naissance & mort entre les temps "tdeb"
% et "tfin", en partant d'une population initiale de "Xdeb" femelles et "Ydeb"
% mâles. Elle affiche, sur un même graphique, les tracés des courbes de la
% population resp. femelle, mâle et totale ; et ne renvoie rien.
function simulation (tdeb, Xdeb, Ydeb, tfin)
% Initialisation des paramètres globaux.
global alphaX alphaY beta0 beta1 betaY subdivisions
init_PNM ();
% "X_" et "Y_" sont les vecteurs destinés à contenir les valeurs de la
% population à différents temps pour le tracé. Ici nous choisissons la stratégie
% consistant à décider à l'avance de l'échantillon des temps pour lesquels nous
% allons tracer les valeurs de la population, avec "subdivisions" subdivisions
% régulières entre "tdeb" et "tfin" (le nombre de subdivisions étant en général
% au moins de l'ordre de 10^3 pour avoir un tracé raisonnablement précis) :
% l'avantage de cette méthode est que nous serons sûrs de ne pas obtenir un
% graphique trop compliqué à tracer (voire de saturer la mémoire de Matlab) si
% la simulation est demandée sur une très longue période de temps. Il y a donc
% subdivisions+1 cases à allouer pour les vecteurs "X_" et "Y_".
X_ = zeros (subdivisions + 1, 1);
Y_ = zeros (subdivisions + 1, 1);
% "pas" est la plage de temps couverte par chaque subdivision.
pas = (tfin - tdeb) / subdivisions;
% "X" et "Y" sont les populations respectives de femelles et de mâles, qu'on
% initialise selon les paramètres fournis en entrée ; et on renseigne la
% première case de "X_" et "Y_" en conséquence.
X = Xdeb;
Y = Ydeb;
X_(1) = X;
Y_(1) = Y;
% Notre simulation va se découper en subdivisions étapes successives, chacune
% correspondant à une subdivision de temps, l'identité de cette subdivision
% étant indiquée par l'indice "i".
for i = 1 : subdivisions
  % Dans ce code, "t" n'indique pas le temps dans l'absolu, mais le temps écoulé
  % depuis le début de la subdivision actuelle.
  t = 0;
  while true
    % On calcule le taux de naissance global, indiqué par "taux_naiss".
    if alphaX * X < alphaY * Y
      taux_naiss = alphaX * X;
    else
      taux_naiss = alphaY * Y;
    end
    % On calcule les taux de morts respectifs pour l'ensemble des femelles et
    % l'ensemble des mâles, indiqués par "taux_mort_X" et "taux_mort_Y".
    taux_mort_X = (beta0 + beta1 * (X + Y)) * X;
    taux_mort_Y = (beta0 + beta1 * (X + Y) + betaY * (Y - 1)) * Y;
    % "taux_tot" est le taux total auquel il se produit un phénomène qui fait
    % évoluer la population.
    taux_tot = taux_naiss + taux_mort_X + taux_mort_Y;
    % Si le taux total des phénomènes est nul, il ne va plus rien se produire ;
    % par conséquent on passe directement à l'étape suivante de la simulation
    % (ce qui évite ainsi la manipulation de quantités infinies).
    if taux_tot == 0;
      break
    else
      % On avance le temps jusqu'au premier instant auquel il arrive quelque
      % chose, ce qui se produit après une loi exponentielle de paramètre
      % taux_tot ; cependant, si on dépasse la fin de notre subdivision, on
      % retient juste qu'il ne se passe plus rien pendant cette subdivision, et
      % on sort de la boucle "while" pour passer à la prochaine subdivision.
      t = t - log (rand ()) / taux_tot;
      if t > pas
        break;
      end
    end
    % Sachant qu'il se passe quelque chose, il reste à savoir quoi… On a quatre
    % cas possibles dont on connait la probabilité pour chacun ; on va donc
    % tirer une loi uniforme "r" entre 0 et 1 et choisir le phénomène qui se
    % produit en fonction de la valeur de r. Selon le phénomène qui se produit,
    % on mettra à jour les valeurs de "X" et "Y" en conséquence.
    r = rand;
    % Premier cas : naissance d'une femelle, si r était entre 0 et
    % taux_naiss/2/taux_tot.
    r = r - taux_naiss / 2 / taux_tot;
    if r < 0
      X = X + 1;
    else
      % Second cas : naissance d'un mâle, si r était entre taux_naiss/2/taux_tot
      % et taux_naiss/taux_tot.
      r = r - taux_naiss / 2 / taux_tot;
      if r < 0
        Y = Y + 1;
      else
        % Troisième cas : mort d'une femelle, si r était entre
        % taux_naiss/taux_tot et (taux_naiss+taux_mort_X)/taux_tot.
        r = r - taux_mort_X / taux_tot;
        if r < 0
          X = X - 1;
        else
          % Dernier cas : mort d'un mâle, si r était entre
          % (taux_naiss+taux_mort_X)/taux_tot et 1.
          Y = Y - 1;
        end
      end
    end
  end
  % Lorsqu'on arrive à la fin d'une subdivision, on stocke dans les vecteurs
  % "X_" et "Y_" les valeurs respectives de X_t et Y_t à ce moment-là.
  X_(i + 1) = X;
  Y_(i + 1) = Y;
end
% Maintenant qu'on a terminé la simulation à proprement parler, il faut encore,
% en vue du tracé, créer un vecteur "XY_" recensant les différentes valeurs de
% la population totale aux temps échantillonnés, ainsi qu'un vecteur "t_"
% contenant les temps échantillonnés eux-mêmes. Le remplissage de ces vecteurs
% est trivial.
XY_ = zeros (subdivisions + 1, 1);
t_ = zeros (subdivisions + 1, 1);
for i = 0 : subdivisions
  XY_(i + 1) = X_(i + 1) + Y_(i + 1);
  t_(i + 1) = tdeb + i * pas;
end
% On trace les trois graphiques sur la même figure (femelles en magenta, mâles
% en cyan, total en noir).
figure (1);
clf;
hold on
plot (t_, X_, 'magenta');
plot (t_, Y_, 'cyan');
plot (t_, XY_, 'black');
hold off
% On ajuste les limites en abscisses de la figure.
xlim ([tdeb, tfin])
% On force la limite inférieure en ordonnées à valoir zéro.
yl = ylim;
yl(1) = 0;
ylim (yl);
return
end
