%% Simulation de la survie de l'espèce sous l'échantillonnage préférentiel.
% Cette fonction fonctionne comme "simulation_muette", mais avec deux
% différences essentielles : d'une part, le paramètre beta_0 est remplacé par
% une valeur "beta0bis" indiquée en entrée par l'opérateur ; d'autre part, outre
% le booléen indiquant la survie ou non de l'espèce, la fonction renvoie la
% densité relative de la loi préférentielle par rapport à la loi authentique
% (pour la simulation qu'on vient de réaliser). Je ne commente que les
% différences par rapport à "simulation_muette".
function [survie, densite] = simulation_muette_pref (Xdeb, Ydeb, T, beta0bis)
global alphaX alphaY beta0 beta1 betaY
X = Xdeb;
Y = Ydeb;
t = 0;
% "densite" est la densité relative entre les deux lois de probabilité, qu'on
% mettra à jour au fur et à mesure.
densite = 1;
% À cause de la façon dont la densité se calcule, on ne pourra plus sortir de la
% boucle aussi simplement que dans "simulation_muette" ; nous introduisons donc
% un indicateur "fin" qui dira si la simulation est terminée ou non.
fin = false;
while true
  if alphaX * X < alphaY * Y
    taux_naiss = alphaX * X;
  else
    taux_naiss = alphaY * Y;
  end
  % Outre les taux de mort sous la loi préférentielle, nous avons besoin de
  % connaitre les taux de mort qu'il y aurait eu sous la loi originale, dont
  % l'expression intervient pour le calcul de la densité : ces taux virtuels
  % sont indiqués par le suffixe '_orig'.
  taux_mort_X = (beta0bis + beta1 * (X + Y)) * X;
  taux_mort_X_orig = (beta0 + beta1 * (X + Y)) * X;
  taux_mort_Y = (beta0bis + beta1 * (X + Y) + betaY * (Y - 1)) * Y;
  taux_mort_Y_orig = (beta0 + beta1 * (X + Y) + betaY * (Y - 1)) * Y;
  taux_tot = taux_naiss + taux_mort_X + taux_mort_Y;
  taux_tot_orig = taux_naiss + taux_mort_X_orig + taux_mort_Y_orig;
  % "attente" est le temps avant qu'il se passe quelque chose. En général, ce
  % sera simplement une loi exponentielle ; cependant il faut tronquer cette loi
  % si cela nous amène à dépasser le temps T. En outre, si la population est
  % éteinte, tous les taux tombent à zéro, auquel cas on va indiquer directement
  % la valeur de "attente" pour éviter des problèmes avec les infinis.
  if taux_tot == 0
    % Dans le cas où il ne se passe plus rien, on arrive évidemment directement
    % à la fin de la simulation.
    attente = T - t;
    fin = true;
  else
    attente = -log (rand ()) / taux_tot;
    t = t + attente;
    % Si on a dépassé le temps T, on corrige la valeur de "attente" et on
    % bascule le marqueur "fin".
    if t > T;
      attente = attente - (t - T);
      fin = true;
    end
  end
  % On met à jour la densité pour tenir compte de ce qui s'est passé au cours de
  % cette étape de simulation, sauf le changement éventuel de population.
  densite = densite * exp (attente * (taux_tot_orig - taux_tot));
  % On sort de la boucle si le marqueur "fin" est actif.
  if fin
    break
  end
  % On va maintenant regarder quel phénomène exactement s'est produit à la fin
  % de cette étape de simulation, et selon le phénomène, on finira de mettre à
  % jour la densité en conséquence.
  r = rand;
  r = r - taux_naiss / taux_tot / 2;
  if r < 0
    X = X + 1;
    % Dans le cas d'une naissance, la densité n'est en fait pas modifiée par
    % cette naissance.
  else
    r = r - taux_naiss / taux_tot / 2;
    if r < 0
      Y = Y + 1;
    else
      r = r - taux_mort_X / taux_tot;
      if r < 0
        X = X - 1;
        densite = densite * taux_mort_X / taux_mort_X_orig;
      else
        Y = Y - 1;
        densite = densite * taux_mort_Y / taux_mort_Y_orig;
      end
    end
  end
end
survie = X > 0 || Y > 0;
return
end
